#if 0 /*
# -----------------------------------------------------------------------
# PhoneInput.py - A class handling multitap input from a remote
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: PhoneInput.py,v 1.4 2004/05/19 13:00:53 viggo Exp $
# -----------------------------------------------------------------------
# $Log: PhoneInput.py,v $
# Revision 1.4  2004/05/19 13:00:53  viggo
# Total rework of the input
#
# Revision 1.3  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# imports
import rc, time, config
from event import *

class PhoneInput:
    """
    This class is a handler for different kinds of input:
     1. Mobilephone style for remote users
     2. Raw text input for keyboard users
    """
    dInput = {}

    # default keymap for remote input
    #  - this is translatable with gettext
    KEY = [
            _(' |+|&|@|/|\\'),
            _('-|?|!|,|.|:|;|<|=|>|(|)|_|1'),
            _('a|b|c|2'),
            _('d|e|f|3'),
            _('g|h|i|4'),
            _('j|k|l|5'),
            _('m|n|o|6'),
            _('p|q|r|s|7'),
            _('t|u|v|8'),
            _('w|x|y|z|9')
          ]

    def __init__(self, parent):

        if hasattr(config, 'MAIL_INPUT_KEYBOARD') \
              and config.MAIL_INPUT_KEYBOARD == 1:
            self.input_mode = 'keyboard'
        else:
            k = 0
            for v in self.KEY:
                self.dInput[k] = v.split('|')
                k += 1
            self.input_mode = 'remote'

        self.parent     = parent
        self._initialize_()


    def _initialize_(self):
        rc.app(self)
        rc.set_context('input')

        self.timer     = 0
        self.timeout   = 1
        self.selected  = 0
        self.position  = 0
        self.text      = ['']
        self.last_key  = None
        self.uppercase = True

    def _getChar_(self, key, index):
        char = self.dInput.get(key)[index]
        if self.input_mode == 'remote':
            if self.uppercase: char = char.upper()
            else:              char = char.lower()

        return char

    def _set_char_(self, char):
        if self.position <= (len(self.text)-1):
            self.text.insert(self.position, char)
        else:
            self.text[self.position] = char
        self.position += 1

    def _multitap_(self, key):
        now = time.time()

        if self.timer == 0:
            self.timer = now

        if key != self.last_key:
            # move to next char and start new timer
            self.last_key = key
            self.selected = 0

            char = self._getChar_(self.last_key, 0)
            self._set_char_(char)

        elif ((now-self.timer)>self.timeout):
            # move to next char
            char = self._getChar_(self.last_key, self.selected)
            self._set_char_(char)

            self.last_key = key
            self.selected = 0
        else:
            # update current char
            self.last_key  = key
            self.selected += 1

            if self.selected>(len(self.dInput[key])-1):
                self.selected = 0

            char = self._getChar_(key, self.selected)
            if self.uppercase: char = char.upper()
            else:              char = char.lower()
            self.text[self.position-1] = char

        self.timer = now

    def _reset_(self):
        self.last_key = '123123'
        self.selected = 0
        self.timer    = 0

    def exit(self):
        # quits the app
        self._initialize_()
        rc.app(None)

    def toggle_upper(self):
        # toggle inputchars casing
        if self.uppercase: self.uppercase = False
        else:              self.uppercase = True

    def move_left(self):
        self._reset_()
        if self.position > 0:
            self.position -= 1

    def move_right(self):
        self._reset_()
        if self.position < len(self.text) - 1:
            self.position += 1

    def insert_newline(self):
        # insert a linebreak
        self._reset_()
        self._set_char_(u'\n')

    def remove_char(self, backspace=True):
        # erase character
        self._reset_()
        if self.position > 0 and backspace:
            self.position -= 1
            del self.text[self.position]
        elif self.position < len(self.text) - 1:
            del self.text[self.position]

    def getInputString(self):
        # get the text input
        t = self.text[0:]
        t.insert(self.position,'|')
        return Unicode(''.join(t))

    def getString(self):
        # get the text without caret
        return Unicode(''.join(self.text))

    def getStringArray(self):
        # get the text as array without caret
        return self.text

    def getInputStringArray(self):
        # get the text as array with caret
        t = self.text
        t.insert(self.position,'|')
        return t

    def getLength(self):
        # get length of textarray (without caret)
        return len(self.text)

    def getCaretPosition(self):
        # get current caret position
        return self.position

    def eventhandler(self, event, menuw=None, arg=None):

        if self.input_mode == 'keyboard':
            if event == BUTTON and event.arg:
                if event.arg == u'\x08':     # backspace
                    self.remove_char()
                elif event.arg == u'\x7f':   # del
                    self.remove_char(False)
                elif event.arg == u'\t':     # tab
                    self._set_char_(u'    ')
                    self.position += 3
                else:
                    self._set_char_(event.arg)

            elif event in INPUT_ALL_NUMBERS:
                self._set_char_('%i' % event.arg)
        else:
            if event in INPUT_ALL_NUMBERS:
                self._multitap_(event.arg)

        # send all events to the parent
        return self.parent.eventhandler(event, arg=event.arg)

