#if 0 /*
# -----------------------------------------------------------------------
# maildir.py - UNIX maildir implementation
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: maildir.py,v 1.7 2004/05/19 10:23:34 viggo Exp $
# -----------------------------------------------------------------------
# $Log: maildir.py,v $
# Revision 1.7  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

"""
Check this error
Exception in thread MailThread:
Traceback (most recent call last):
  File "/usr/lib/python2.2/threading.py", line 414, in __bootstrap
    self.run()
  File "/usr/lib/python2.2/site-packages/freevo/mail/watcher.py", line 147, in run
    self.checkmail()
  File "/usr/lib/python2.2/site-packages/freevo/mail/watcher.py", line 75, in checkmail
    upd,rem,add,idl = self.conn.getUpdates(f.name,f.seen)
  File "/usr/lib/python2.2/site-packages/freevo/mail/maildir.py", line 145, in getUpdates
    r = self.findmatch(i,seen)
  File "/usr/lib/python2.2/site-packages/freevo/mail/maildir.py", line 120, in findmatch
    d,id,f = re.match('(.*)/(.*):2,(.*)',uid).groups()
AttributeError: 'NoneType' object has no attribute 'groups'
"""


# python modules
import mailbox, email, os, re

# freevo modules
import config

# mailreader modules
from mail         import MailProtocol
from mailutil     import *
from types        import *


class Maildir(MailProtocol):
    """
    UNIX maildir implementation
    Todo:
     Support the Courier spec better. This can provide us with
     logical folders, like imap - and provides flags in the filename.

     - Subfolders of the maildir is named in the following convention:
        - Has zerofile named 'mailfolder' means it is a mailfolder
          with new/cur/tmp-subfolders.
        - .Test is a subfolder of the maildir
        - .Test.03 is a subfolder of .Test

     - The reader does not do anything about the fact that we have
     read the mail. (It should be moved to ./cur/ when read, and
     set :2,S to the filename for courier style maildirs.

    example for naming:
    /home/viggo/.maildir/cur/1074024603.11092_1.genesis.katatonic.org:2,RS


    Docs:
     QMail Spec: http://www.qmail.org/man/man5/maildir.html
     Courier Spec: http://www.courier-mta.org/maildir.html
     doc: http://www.python.org/doc/current/lib/module-mailbox.html
    """
    def __init__(self):
        MailProtocol.__init__(self)
        self.caps = [ 'DEL' ]
        if type(self.file) in StringTypes:
            self.tryCourier()

    def tryCourier(self):
        names = os.listdir(self.file)
        dirs = []
        for name in names:
            if name.startswith('.',0,1):
                candidate = '%s/%s' % (self.file,name)
                if vfs.isdir(candidate):
                    for a in os.listdir(candidate):
                        if a == 'maildirfolder':
                            dirs.append(candidate)
        self.file = dirs

    def _getBox(self,mbox,factory=1):
        if factory == 1:
            return mailbox.Maildir(mbox , self._msgfactory)
        elif factory == 2:
            return mailbox.Maildir(mbox, self._uidfactory)

    def _uidfactory(self,fp):
        uids      = fp.name.split('/')
        uid       = uids[ (len(uids)-1) ]
        subfolder = uids[ (len(uids)-2) ]
        return '%s/%s' % (subfolder,uid)

    def _msgfactory(self,fp):
        try:
            return email.message_from_file(fp), fp.name
        except email.Errors.MessageParseError:
            return ''

    def getuidl(self, folder, split=False):
        uid = []
        suid = []
        app = uid.append
        sapp = suid.append

        mbox = self._getBox(folder,factory=2)
        id = mbox.next()

        while id:
            app(id)
            if split:
                sapp(id.split(':')[0])
            id = mbox.next()

        if split:
            return uid,suid
        else:
            return uid


    def findmatch(self,uid,seen):
        d,id,f = re.match('(.*)/(.*):2,(.*)',uid).groups()

        for i in seen:
            od,oid,of = re.match('(.*)/(.*):2,(.*)',i).groups()
            if oid == id:
                if od!=d:     # changed folder
                    if d=='cur': return i,'seen'
                    else       : return i,'new'
                elif of!=f:   # changed flag
                    if 'S' in f: return i,'seen'
                    else       : return i,'new'
        # new mail
        return False

    def getUpdates(self, folder, seen):
        upd   = [] # flagupdate of header
        rem   = [] # uidlist of msg's to remove
        add   = [] # new message

        uidl,suidl = self.getuidl(folder,split=True)

        # look for new and updated messages
        for i in uidl:
            if i not in seen:
                # needs further investigaion
                r = self.findmatch(i,seen)
                if r:
                    o,f = r
                    upd.append( (o,i,f) )
                else:
                    add.append(self.getHeader(folder,i))
        self.Sort(add)

        # look for removed messages
        for i in seen:
            _i = i.split(':')[0]
            if _i not in suidl:
                rem.append(i)

        return upd,rem,add,uidl

    def _getHeader(self,folder,uid):
        try:
            fp = file('%s/%s' % (folder,uid),'r')
            strHeader = fp.read()
            fp.close()
            header = getHeader(strHeader)
            header.UID = uid
            header.Folder = folder
            return header
        except:
            return BaseHeader()

    def _parseFlags(self,uid):
        """
        try to look for qmail and courier flags
        """
        try:
            f = uid.split(':')[1]
            if f == '2,': return 'new'
            if 'S' in f.split(',')[1]:
                return 'seen'
        except: pass

        flag = uid.split('/')[0]

        if  flag == 'cur': return 'seen'
        else:              return 'new'

    def getHeader(self,folder,uid):
        header = self._getHeader(folder,uid)
        header.Flag = self._parseFlags(uid)
        return header

    def getHeaders(self,folder):
        uid = self.getuidl(folder)
        mails = []

        for id in uid:
            header = self._getHeader(folder,id)
            header.Flag = self._parseFlags(id)
            mails.append(header)

        return self.Sort(mails)

    def getMessage(self,header):
        fname = '%s/%s' % (header.Folder,header.UID)
        try:
            fp = file(fname,'r')
            m = fp.read()
            fp.close()
            return getMessage(m,header)
        except:
            _debug_('Mail: Error reading selected mail')
            return BaseMessage(header)

    def delMessage(self, header):
        fname = '%s/%s' % (header.Folder,header.UID)
        if header.UID in self.getuidl(header.Folder):
            if len(fname)>2 and fname!='/':
                try:
                    os.unlink(fname)
                    return True
                except:
                    _debug_('Error deleting mail: %s' % fname)
        return False
