#if 0 /*
# -----------------------------------------------------------------------
# mailutil.py - utility class for the mailreader
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: mailutil.py,v 1.12 2004/05/19 10:58:02 viggo Exp $
# -----------------------------------------------------------------------
# $Log: mailutil.py,v $
# Revision 1.12  2004/05/19 10:58:02  viggo
# Typo!
#
# Revision 1.11  2004/05/19 10:43:21  viggo
# Make sure to remove some dirty chars in the header
#
# Revision 1.10  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Notes: - Probably possible to speed up
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

#  freevo modules
from mail          import BaseHeader, BaseMessage

# python modules
from HTMLParser    import HTMLParser
from email.Parser  import Parser, HeaderParser
from email.Utils   import *
from email.Header  import *
from email.Message import *
from mimetypes     import *

import time

# add more extensions (use the config?)
MAIL_MEDIA_EXT = [ '.jpg','.png','.bmp', '.gif' ]


# try to sanitize the mail for the gui
def m_sanitize(text=[]):
    text = "".join(text)
    text = text.replace('\r','')
    text = text.replace('\000', '')
    return text


# writes attachement files
def m_writeattachments(part, counter, fname):
    if not fname:
        ext = guess_extension(part.get_type())
        if not ext: ext = '.bin'
        fname = 'part-%03d%s' % (counter, ext)
    else:
        l = len(fname)
        l_0 = l - 4
        ext = fname[l_0:l]
        ext = '%s' % ext

    if ext.lower() in MAIL_MEDIA_EXT:
        try:
            place = vfs.getoverlay('/mail/attachments/%s' % fname)
            fp = vfs.open( place ,'wb')
            fp.write(part.get_payload(decode=1))
            fp.flush()
            fp.close()
            return (place,fname)
        except: pass
    return None


# returns an instance of BaseMessage
def getMessage(message, header, isMessage=0):
    body = []
    b_append = body.append

    counter = 1
    text = False
    images = []

    if isMessage:
        msg = message
    else:
        try:
            msg = Parser().parsestr(message)
        except:
            _debug_('Error parsing the mail!')
            b_append(_('Error parsing mail'))
            msg = False

    if msg:
        isAltern = False

        for part in msg.walk():
            ctype = part.get_content_type()

            if ctype ==  'multipart/alternative':
                isAltern = True

            elif ctype == 'text/plain':
                if text:
                    b_append(_('\n \n::: New text/plain part ::: \n \n'))
                b_append(part.get_payload(decode=1))
                text = True
            elif ctype == 'text/html' and not isAltern:
                try:
                    b_append(MyHTMLParser(part.get_payload(decode=1)).getText())
                except:
                    _debug_('HTMLParser failed')
            elif part.get_content_maintype() != 'multipart':
                fname = part.get_filename()
                img = m_writeattachments(part, counter, fname)
                if img: images.append(img)
                counter += 1

    # create the msg
    msg             = BaseMessage(header)
    msg.BodyText    = m_sanitize(body)
    msg.MediaAttach = images
    msg.unicode()
    return msg


# returns an instance of BaseHeader
def getHeader(header=None, message=True):
    h = BaseHeader()

    if not header: return h

    if message:
        p = HeaderParser().parsestr(header, headersonly=1)
    else:
        p = header

    for part in p.walk():
        h.FromName,h.FromAddr = parseaddr(part['From'])
        h.ToName,h.ToAddr = parseaddr(part['To'])

        if h.FromName == '':
            h.FromName = h.FromAddr
        else:
            h.FromName = decode_header(h.FromName)[0][0]

        if h.ToName == '':
            h.ToName = h.ToAddr
        else:
            h.ToName = decode_header(h.ToName)[0][0]

        h.UID = part['Message-ID']
        h.Subject = decode_header(part['Subject'])[0][0].strip()
        h.Subject = h.Subject.replace('\t',' ')
        h.Subject = h.Subject.replace('\n',' ')
        h.Sort = 0
        h.Date = part['Date']

        date = parsedate_tz(h.Date)

        try:    date = mktime_tz(date)
        except: date = False

        if date:
            h.Sort = date
            h.Date = time.ctime(date)
        h.unicode()
    return h



class MyHTMLParser(HTMLParser):
    def __init__(self,text):
        HTMLParser.__init__(self)
        self.ParsedText = ''
        self.fText = []
        self.feed(text)
        self.close()

    def getText(self):
        return self.ParsedText

    def stripComment(self, data):
        s = data.find('<!--')
        if s != -1:
            e = data.find('-->')
            if e != -1:
                data = data[0:s] + data[(e+3):]
                self.stripComment(data)
        return data

    def handle_data(self, data):
        if self.lasttag in ['title','meta']:
            return

        data = self.stripComment(data)
        data = data.replace('\n','')

        last = len(self.fText)-1
        i = 0
        if last>=0:
            for t in self.fText:
                if i==last:
                    break
                self.ParsedText += t
                i+=1
            self.ParsedText += self.fText[last].replace(' ','') + data
        else:
            self.ParsedText += data
        self.fText = []

    def handle_starttag(self, tag, attrs):
        self.lasttag = tag
        if tag == 'br':
            self.fText.append('\n ')

    def handle_endtag(self, tag):
        self.lasttag = None
        if tag in ['p','div']:
            self.fText.append('\n ')

