#if 0 /*
# -----------------------------------------------------------------------
# mailwriter.py - Makes it possible to reply to mails via freevo
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: mailwriter.py,v 1.18 2004/05/19 13:15:17 viggo Exp $
# -----------------------------------------------------------------------
# $Log: mailwriter.py,v $
# Revision 1.18  2004/05/19 13:15:17  viggo
# Draw arrows in edit area
#
# Revision 1.17  2004/05/19 13:02:45  viggo
# Use the new input.
# Fix a crash.
# Proper movement of the current text area.
#
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# python modules
import smtplib, time, os
from email.Message import Message

# freevo modules
import config
from PhoneInput import PhoneInput
from event import *
from gui  import ConfirmBox

import skin, rc, osd
skin = skin.get_singleton()


mDisclaimer = """

-------------------------------------------------------
This message was sent by the freevo mail plugin.
Get your Freevo from http://www.freevo.org

"""

class MailSender:
    """
    This class creates a skin area used when replying to mail.

    TODO: - Add possibility to send New mail, without predefined Subject and To
    """
    def __init__(self, org_msg, menuw, Subject=None):
        self.Input          = PhoneInput(self)
        self.org_msg        = org_msg
        self.mwText         = MailWriterText()
        self.mwText.dict    = self.Input.dInput
        self.mwText.subject = org_msg.Subject
        self.menuw          = menuw
        skin.draw('mailwriter', self.mwText)

    def eventhandler(self, event, arg=None):
        if event == INPUT_ENTER:
            if self.Input.getLength() != 0:
                self.text = self.Input.getString()
                self.Input.exit()
                ms = _('Do you want to send this message?')
                ConfirmBox(text=ms,handler=self._sendmail_,
                           handler_message=_('Sending message...')).show()

        elif event == INPUT_UP:
            self.Input.remove_char()

        elif event == INPUT_DOWN:
            self.Input.insert_newline()

        elif event == INPUT_LEFT:
            self.Input.move_left()

        elif event == INPUT_RIGHT:
            self.Input.move_right()

        elif event == BUTTON and event.arg == 'd':
            self.mwText.togglemode()
            self.Input.toggle_upper()

        elif event == INPUT_EXIT:
            self.Input.exit()
            self.menuw.show()
            self.menuw.refresh()

        if event not in [INPUT_ENTER, INPUT_EXIT]:
            self.mwText.text = self.Input.getInputString()
            self.mwText.pos  = self.Input.getCaretPosition()
            skin.draw('mailwriter', self.mwText)

        return False

    def _sendmail_(self):
        """
        sends the created message

        Todo: Respect Reply-to from original message
        """
        cfg = config.MAIL_SMTP

        if cfg.has_key('name'):
            frm = '"%s" <%s>' % (cfg['name'], cfg['from'])
        else:
            frm = cfg['from']

        if self.org_msg.FromName != self.org_msg.FromAddr:
            to = '"%s" <%s>' % (self.org_msg.FromName, self.org_msg.FromAddr)
        else:
            to = self.org_msg.FromAddr

        msg = Message()
        msg.set_default_type('text/plain')
        msg.add_header('From', frm)
        msg.add_header('To',to)
        if cfg.has_key('reply-to'): msg.add_header('Reply-To', cfg['reply-to'])
        msg.add_header('Subject', 'Re: %s' % self.org_msg.Subject)
        msg.add_header('X-Mailer', 'Freevo mail 0.6')

        content = String(_('%s \n\n' % self.text))
        content += String(_('On %s %s wrote:\n') % (self.org_msg.Date, self.org_msg.FromName))

        lines = self.org_msg.BodyText.split('\n')
        for l in lines:
            content += String(_('>%s\n') % l)

        if cfg.has_key('sig'):
            content += String(_('\n\n%s') % cfg['sig'])

        content += String(_(mDisclaimer))

        msg.set_payload(content)

        if msg:
            server = smtplib.SMTP()
            if cfg.has_key('port'):
                server.connect(cfg['server'], cfg['port'])
            else:
                server.connect(cfg['server'])

            if cfg.has_key('username') and cfg.has_key('password'):
                server.login(cfg['username'],cfg['password'])

            dRefused = None

            try:
                dRefused = server.sendmail(cfg['from'], self.org_msg.FromAddr, msg.as_string())
            except Exception,err:
                _debug_(err)

            _debug_(dRefused,2)
            server.quit()


class MailWriterText:
    text    = ''
    pos     = 0
    mode    = _('Caps')
    subject = _('No subject')

    keys = [( _('Enter'),      _('Send message')),
            ( _('Display'),    _('Textmode')    ),
            ( _('Up'),         _('Erase')       ),
            ( _('Down'),       _('Linebreak')   ),
            ( _('Left/Right'), _('Move caret')  )
           ]

    def togglemode(self):
        if self.mode == _('Caps'):
            self.mode = _('Small')
        else:
            self.mode = _('Caps')



class MailWriter(skin.Area):
    def __init__(self):
        skin.Area.__init__(self, 'writer')

        # reader display fonts
        self.sub_font  = skin.get_font('small0')
        self.stat_font = skin.get_font('small1')

        self.up   = os.path.join(config.ICON_DIR, 'up.png')
        self.down = os.path.join(config.ICON_DIR, 'down.png')

    def update_content(self):
        self.content = self.calc_geometry(self.layout.content, copy_object=True)

        ## TODO: scroll with position
        x    = self.content.x + 10
        y    = self.content.y + 5
        maxw = self.content.width - 20

        ## draw information
        txt = u'Re: %s' % self.menu.subject

        self.drawstring(txt,self.stat_font,  self.content, x, y,
                        height=self.stat_font.height, width=maxw, mode='soft')

        ## draw the text
        y += self.stat_font.height

        # split the text and draw only what's necessery
        text  = self.menu.text
        lines = text.split('\n')

        ll    = len(lines)

        maxh = self.content.height - 20
        maxl = maxh/self.sub_font.height

        if ll > maxl:
            tail  = len(text[self.menu.pos:].split('\n'))
            front = len(text[:self.menu.pos].split('\n')) + 1
            subl  = front - tail

            start = max(0,front-maxl)
            stop  = start + maxl
            txt = '\n'.join(lines[start:stop])

            # draw arrows
            _x  = self.content.x + self.content.width - 25
            _y0 = self.content.y + 5 + self.stat_font.height
            _y1 = self.content.y + self.content.height - 25

            if start > 0:
                self.drawimage(self.up,   ( _x, _y0, 20, 20))
            if stop < ll:
                self.drawimage(self.down, ( _x, _y1, 20, 20))

        else:
            txt = text

        self.drawstring(txt, self.sub_font, self.content,
                        x, y, height=maxh, width=maxw,
                        mode='soft')



class MailWriterLegend(skin.Area):
    def __init__(self):
        skin.Area.__init__(self, 'legend')
        self.tfont = skin.get_font('small0')
        self.font  = skin.get_font('small1')

        if hasattr(config, 'MAIL_INPUT_KEYBOARD') \
              and config.MAIL_INPUT_KEYBOARD == 1:
            self.input_mode = 1
        else:
            self.input_mode = 0

    def update_content(self):
        # TODO: only draw when something has changed
        self.content = self.calc_geometry(self.layout.content, copy_object=True)

        if self.input_mode == 1:
            self.update_keyboard()
        else:
            self.update_remote()


    def update_remote(self):
        """
        Drawmethod for remote input
        """
        x = self.content.x + 10
        y = self.content.y + 5

        for (k,v) in self.menu.dict.items():
            self.drawstring('%s' % Unicode(k), self.tfont,self.content,
                            x, y, height=self.tfont.height,mode='hard')
            tx = x
            for val in v:
                val = Unicode(val)
                if self.menu.mode==_('Caps'): val = val.upper()
                tx += 1.5*self.font.stringsize('a')
                self.drawstring(val, self.font, self.content,
                                tx, y,height=self.font.height,mode='hard')
            y += self.font.height

        # draw other keys info
        x = self.content.width/2 + self.content.x + 50
        y = self.content.y + 5
        for (k,v) in self.menu.keys:
            self.drawstring(Unicode(k), self.tfont,self.content,
                            x, y, height=self.tfont.height,mode='hard')

            self.drawstring(Unicode(v), self.font,self.content,
                            x+110, y, height=self.font.height,mode='hard')
            y += self.font.height


    def update_keyboard(self):
        """
        Drawmethod for keyboard input
        """

        # draw other keys info
        x = self.content.width/2 + self.content.x + 50
        y = self.content.y + 5
        i = 0
        for (k,v) in self.menu.keys:
            if i == 1:
                i += 1
                continue
            self.drawstring(Unicode(k), self.tfont,self.content,
                            x, y, height=self.tfont.height,mode='hard')

            self.drawstring(Unicode(v), self.font,self.content,
                            x+110, y, height=self.font.height,mode='hard')
            y += self.font.height
            i += 1


skin.register ( 'mailwriter', ('screen', MailWriter(), MailWriterLegend(), 'plugin'))
