#if 0 /*
# -----------------------------------------------------------------------
# pop3.py - POP3 protocol implementation
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: pop3.py,v 1.9 2004/05/19 10:23:34 viggo Exp $
# -----------------------------------------------------------------------
# $Log: pop3.py,v $
# Revision 1.9  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Todo:
#       - different auth methods, APOP,RPOP etc?
#       - UIDL and TOP may not be implemented in servers! (marked as opt)
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# python modules
import poplib, string, time

# freevo modules
import config

# mailreader modules
from mail     import MailProtocol
from mailutil import *



class Pop3(MailProtocol):
    """
    Pop3 implementation
    doc: http://www.python.org/doc/current/lib/pop3-objects.html
         http://www.isi.edu/in-notes/rfc1939.txt
    """
    def __init__(self):
        MailProtocol.__init__(self)
        self.caps = [ 'DEL' ]

    def logout(self):
        try:
            self.M.quit()
        except poplib.error_proto, err:
            _debug_('Pop3: Error on logout.\n%s' % err)

    def login(self):
        try:
            self.M = poplib.POP3(self.host)
            self.M.user(self.username)
            self.M.pass_(self.password)
            return 'OK'
        except Exception, err: # seems to to raise socket.error too
            _debug_('Pop3: Error on login.\n%s' % err)
            return '%s' %err

    def _check(self,resp):
        """
        check server response
        """
        if resp.startswith('+OK',0,3):
            return True
        return False

    def _getMsgNum(self,uid):
        """
        returns the pop3 msgnum given its uid
        """
        try:
            response,aid,octets = self.M.uidl()

            for a in aid:
                if a.split()[1] == uid:
                    return a.split()[0]
        except poplib.error_proto, err:
            _debug_('Pop3: Error getting msgnumber.\n%s' % err)
            return False
        return False

    def getFolders(self):
        """
        Pop3 has no 'Folder' option
        Just return some intuitive names
        """
        return [ 'Pop3 mail' ]

    def getHeaders(self,folder=None):
        """
        this function seems to take a lot of time
        try to speed this up
        """
        uid = []
        msgnum = []
        mails = []
        try:
            response,aid,octets = self.M.uidl()

            for a in aid:
                uid.append(a.split()[1])
                msgnum.append(a.split()[0])
            if self._check(response):
                for i in range(len(uid)):
                    lines = self.M.top(msgnum[i],0)[1]
                    head = string.join(lines,'\n')
                    header = getHeader(head)
                    header.Folder = 'Pop3 mail'
                    header.UID = uid[i]

                    # pop3 has no flags, so we manually set it to true
                    # correct me if this is wrong
                    header.Flag = 'new'
                    mails.append(header)
                return self.Sort(mails)
        except poplib.proto_error, err:
            _debug_('Pop3: Error when getting headers\n%s' % err)

        return []

    def getHeader(self, folder, uid):
        try:
            response,aid,octets = self.M.uidl()
            msgnum = -1

            if self._check(response):
                for a in aid:
                    if string.split(a,' ')[1] == uid:
                        msgnum = string.split(a,' ')[0]
                        break
                if msgnum != -1:
                    head          = string.join(self.M.top(msgnum,0)[1],'\n')
                    header        = getHeader(head)
                    header.UID    = uid
                    header.Flag   = 'new'
                    header.Folder = 'Pop3 mail'
                    return header

        except poplib.error_proto, err:
            _debug_('Pop3: Error getting header.\n%s' % err)

        return BaseHeader()

    def getuidl(self,folder):
        uid = []
        try:
            response,aid,octets = self.M.uidl()

            if self._check(response):
                for id in aid:
                    uid += [ string.split(id,' ')[1] ]
            return uid
        except poplib.error_proto, err:
            _debug_('Pop3: Error getting uidlist\n%s' % err)

        return []

    def getUpdates(self, folder, seen):
        upd  = []
        rem  = []
        add  = []
        uidl = self.getuidl(folder)

        for i in uidl:
            if i not in seen:
                add.append(self.getHeader(folder,i))

        for i in seen:
            if i not in uidl:
                rem.append(i)

        return upd,rem,add,uidl


    def getMessage(self,header):
        uid = self._getMsgNum(header.UID)
        if uid:
            try:
                lines = self.M.retr(uid)[1]
                return getMessage(string.join(lines,'\n'),header)
            except poplib.error_proto, err:
                _debug_('Pop3: Error getting message\n%s' % err)
        return

    def delMessage(self,header):
        try:
            uid = self._getMsgNum(header.UID)
            if uid:
                self.M.dele(uid)
                self.logout()
                self.login()
        except poplib.error_proto, err:
            _debug_('Pop3: Error deleting message\n%s' % err)
            return False

        return True

    def purgeFolder(self,folder):
        try:
            response,uidl,octets = self.M.uidl()
            if self._check(response):
                for id in uidl:
                    self.M.dele(id)
                self.logout()
                self.login()
        except poplib.error_proto, err:
            _debug_('Pop3: Error deleting messages\n%s' % err)
            return False

        return True

