#if 0 /*
# -----------------------------------------------------------------------
# watcher.py - Mailwatcher for freevo
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: watcher.py,v 1.15 2004/05/19 10:23:34 viggo Exp $
# -----------------------------------------------------------------------
# $Log: watcher.py,v $
# Revision 1.15  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# freevo modules
import plugin, config, rc, osd , util.fileops
from gui   import AlertBox
from event import *
from mail  import *

# python modules
import threading



class MailThread(threading.Thread):
    Connected   = True
    Starting    = True
    Result      = None
    ActionQueue = []

    def __init__(self, caller=None):
        threading.Thread.__init__(self)
        self.Ready = threading.Condition()
        self.Busy  = threading.Condition()
        self.setName('Freevo mailthread')
        self.setDaemon(1)

        self.mwatch = caller
        self.conn   = get_protocol()

    def osdNotify(self,num=0):
        if num>0:
            msg = _('%i new mail(s) arrived') % num
            rc.post_event(Event(OSD_MESSAGE, arg=msg))

    def setAction(self, action, arg=None, folder=None):
        # we don't support queueing atm
        if len(self.ActionQueue)!=0 and action=='check' or self.Starting:
            return
        self.ActionQueue.append( { 'action' : action,
                                   'arg'    : arg,
                                   'folder' : folder } )
    def checkmail(self):
        new_m = 0
        for f in self.mwatch.FDict.values():
            upd, rem, add, f.seen = self.conn.getUpdates(f.name, f.seen)
            new_m += f.Update(rem, add, upd)
        self.osdNotify(new_m)

    def checkfolder(self, folder):
        upd, rem, add, folder.seen = self.conn.getUpdates(folder.name, folder.seen)
        nm = folder.Update(rem, add, upd)
        self.osdNotify(nm)

    def startup(self):
        d = self.mwatch.FDict
        has_key = d.has_key

        for f in self.conn.getFolders():
            if not has_key(f):
                d[f] = Folder(f, self.conn.getHeaders(f))

    def read(self, header, folder):
        cached = True
        ret = folder.load_Message(header.UID)
        if not ret and self.conn.login() == 'OK':
            self.Connected = True
            cached = False
            ret = self.conn.getMessage(header)
            self.conn.logout()
            folder.save_Message(ret)

        if header.Flag == 'new':
            if 'FLAG' in self.conn.caps and cached and self.conn.login() == 'OK':
                self.Connected = True
                self.conn.flagMessage(header)
                self.conn.logout()
            folder.Update( update=[(header.UID, header.UID,'seen')] )
        return ret

    def getResult(self):
        return self.Result

    def run(self):
        self.mwatch.Wait.acquire()
        self.perform({'action' : 'start' })

        # try to connect with poll_interval
        # if initially disconnected
        while(not self.Connected):
            self.Ready.acquire()
            self.Ready.wait()
            self.Ready.release()
            self.perform({'action' : 'start' })

        self.Starting = False

        while(1):
            self.Ready.acquire()
            #print '> Main wait'
            #print '> releasing'
            self.mwatch.Wait.release()
            #print '> released'
            self.Ready.wait()
            self.Ready.release()
            #print '< Main wait over'
            self.perform(self.ActionQueue.pop(0))
            #print '< performing over'
            #print '> notify'
            self.Busy.notify()

    def perform(self, action):
        ret   = None

        if action['action']=='READ':
            ret = self.read(action['arg'], action['folder'])
        elif self.conn.login()=='OK':
            self.Connected = True

            if action['action']=='check':
                self.checkmail()

            elif action['action']=='start':
                self.startup()

            elif action['action']=='CHECK FOLDER':
                self.checkfolder(action['folder'])

            elif action['action']=='MARK UNREAD':
                ret = self.conn.flagMessage(action['arg'],'UNREAD')
                if action['arg'].Flag != 'new':
                    action['folder'].Update( update=[(action['arg'].UID,
                                             action['arg'].UID, 'new')] )

            elif action['action']=='MARK READ':
                ret = self.conn.flagMessage(action['arg'],'READ')
                if ret and action['arg'].Flag == 'new':
                    action['folder'].Update( update=[(action['arg'].UID,
                                             action['arg'].UID, 'seen')] )

            elif action['action']=='DELETE':
                ret = self.conn.delMessage(action['arg'])
                if ret: action['folder'].Update(remove=[action['arg'].UID])

            elif action['action']=='MARK ALL READ':
                ret = self.conn.markAllRead(action['arg'])
                if ret:
                    action['folder'].Update(remove=[action['folder'].seen])

            elif action['action']=='UNSUBSCRIBE':
                f = action['arg']
                if self.conn.unsubscribe(f):
                    fd = self.mwatch.FDict
                    new_fdict = {}
                    for (k,v) in fd.items():
                        if k != f: new_fdict[k] = v

                    self.mwatch.FDict = new_fdict
                    ret = True

            elif action['action']=='SUBSCRIBE':
                if self.conn.subscribe(action['arg']):
                    h = self.conn.getHeaders(action['arg'])
                    self.mwatch.FDict[action['arg']] = Folder(action['arg'], h)
                    ret = True

            elif action['action']=='GETUNSUB':
                ret = self.conn.getUnsubscribed()

            self.conn.logout()
        else:
            self.Connected = False

        self.Result = ret



class Mailwatcher(plugin.DaemonPlugin):
    """
    Activated by the mail plugin.
     Keeps a cache of headers and messages
     and checks for updates in mailfolder.
    """

    class error(Exception):
        errd  = { 'offline'     : _('Server unavailable. Try again later.'),
                  'off_warn'    : _('Server unavailable. Mail may not be up to date.'),
                  'off_nocache' : _('Server offline. No cache was found.'),
                  'off_nocmd'   : _('Server offline. Command not available.'),
                  'failed'      : _('The requested command failed.'),
                  'start'       : _('Initializing folders. Try again later.') }

        def __init__(self, value):
            value = self.errd.get(Unicode(value),
                                  _('Undefined error occured') )
            self.alert = AlertBox(text=value)

    Disabled   = False
    Online     = True
    FDict      = {}

    def __init__(self):
        plugin.DaemonPlugin.__init__(self)

        self.type       = config.MAIL_TYPE.lower()
        self.Wait       = threading.Lock()

        try:    self.poll_interval = config.MAIL_INTERVAL*100
        except: self.poll_interval = 30000

        # delete old cache
        d = vfs.getoverlay('/mail')
        if vfs.isoverlay(d):
            util.fileops.rmrf(d)

        self.mthread = MailThread(self)
        self.mthread.start()
        self.caps = self.mthread.conn.caps
        plugin.register(self, 'Mailwatcher')

    def consume(self, action, arg=None, folder=None, wait=1):
        busy  = self.mthread.Busy
        ready = self.mthread.Ready

        if self.Wait.acquire(wait):
            #print '=============== acquired wait: %i' %wait
            self.mthread.setAction(action, arg, folder)
            ready.acquire()
            ready.notify()
            ready.release()
            if wait==1:
                #print '# Waiting in consume'
                self.mthread.Busy.acquire()
                self.mthread.Busy.wait()
                ret = self.mthread.getResult()
                self.mthread.Busy.release()
                #print '# Waiting in consume over'
                return ret

    def _busyIcon(self):
        if config.OSD_BUSYICON_TIMER:
            osd.get_singleton().busyicon.wait(config.OSD_BUSYICON_TIMER[0])

    def getFolders(self):
        if self.mthread.Starting:
            if self.mthread.Connected:
                raise self.error('start')
            else:
                raise self.error('offline')
        s = self.FDict.keys()
        s.sort(lambda l, o: cmp(l.upper(), o.upper()))
        return s

    def getNewMailCount(self):
        if self.mthread.Connected:
            count = 0
            for f in self.FDict.values():
                count += f.new
        else: count = -1

        return count

    def getHeaders(self, folder, offline=False):
        if not self.mthread.Connected and not offline:
            raise self.error('off_warn')

        self._busyIcon()
        # load from cache
        f = self.FDict.get(folder)
        if self.mthread.Starting: return []
        elif f:                   return f.load_Headers()
        else:                     raise self.error('off_nocache')

    def action(self, action=None, arg=None):
        try:    f = arg.Folder
        except: f = arg

        if action=='STAT':
            return self.FDict.get(f).getCount()
        else:
            self._busyIcon()
            ret = self.consume(action, arg, self.FDict.get(f), wait=1)

            if ret: return ret
            else:   raise self.error('failed')

    def poll(self):
        if not self.mthread.Starting:
            self.consume('check',wait=0)
        else:
            self.mthread.Ready.notify()



class Folder:
    """
    helper class for a folder
    Todo: Finish cachehandling for msg and folder
    """
    new, count, seen = 0,0, []

    def __init__(self, name='', headers=[]):
        self.name  = name
        self.fname = '/mail/%s/header-cache' % name.replace('/', '.')

        app = self.seen.append
        for h in headers:
            app(h.UID)
            if h.Flag == 'new':
                self.new += 1
        self.count = len(headers)
        self.save_Headers(headers)

    def getCount(self):
        return self.new, self.count

    def load_Headers(self):
        ret = []
        if vfs.isfile(self.fname):
            ret = util.fileops.read_pickle(self.fname)
            ret.pop(0)
        return ret

    def save_Headers(self, headers=[]):
        headers.insert(0,'top')
        util.fileops.save_pickle(headers,self.fname)

    def save_Message(self, msg=None):
        id = msg.UID.replace('/','.')
        nm = self.name.replace('/','.')
        fname = '/mail/%s/%s' % (nm, id)
        util.fileops.save_pickle(msg, fname)

    def rm_Message(self, uid):
        d = vfs.getoverlay('/mail')
        id = uid.replace('/','.')
        file = '%s/%s/%s' % (d, self.name.replace('/','.'), id)
        if vfs.isfile(file):
            vfs.unlink(file)

    def load_Message(self, uid):
        id = uid.replace('/','.')
        nm = self.name.replace('/','.')
        fname = '/mail/%s/%s' % (nm, id)
        if vfs.isfile(fname):
            return util.fileops.read_pickle(fname)
        else:
            return False

    def Update(self, remove=[], add=[], update=[]):
        """
        Updates an entire folder
        """
        newm  = 0
        tnewm = 0

        if len(remove)>0 or len(add)>0 or len(update)>0:
            org_headers = self.load_Headers()
            new_headers = []
            app = new_headers.append
            d_upd = {}

            # update headers
            for (org,uid,f) in update:
                d_upd[org] = (uid,f)

            # append newly discovered headers
            for h in add:
                app(h)
                if h.Flag == 'new':
                    newm  += 1
                    tnewm += 1

            # update and remove
            for h in org_headers:
                if h.UID not in remove:
                    if h.UID in d_upd.keys():
                        i,f = d_upd[h.UID]
                        if f == 'new' and h.Flag != 'new':
                            newm += 1
                        if i != h.UID: h.UID = i
                        h.Flag = f
                    if h.Flag == 'new': tnewm += 1
                    app(h)
                else:
                    self.rm_Message(h.UID)

            self.count = len(new_headers)
            self.new   = tnewm
            self.save_Headers(new_headers)

        return newm
